@@ -1,7 +1,40 @@
-#define MOD_LISP_VERSION "2.33"
+#define MOD_LISP_VERSION "2.38"
 #define HEADER_STR_LEN 500
 
 /*
+  Version 2.41
+  Case insensitive version of the set-cookie
+
+  Version 2.40
+  Allow more than one Set-Cookie
+  (it was added in 2.2 but was wrongly removed in 2.35)
+
+  Version 2.39
+  Case insensitive parsing of the Lisp header names
+  (for compatibility with mod_lisp2)
+
+  Version 2.38
+  New "server-baseversion" and "modlisp-version" headers
+  (Edi Weitz)
+
+  Version 2.37
+  Create new socket (instead of reusing) if IP/port combo has changed
+  (Edi Weitz)
+
+  Version 2.36
+  Close Lisp socket (and buffer) if connection is aborted.
+  Some cleanup.
+  (Edi Weitz)
+
+  Version 2.35
+  Moved back the LispSocket and UnsafeLispSocket variables as global variables 
+  instead of config struct variables. The struct is reset at each new request
+  so the sockets were lost instead of reused.
+  (Found and fixed by Edi Weitz)
+
+  Version 2.34
+  Send the SCRIPT_FILENAME variable to Lisp when it is there.
+
   Version 2.33
   Added a couple of new headers like "Log-Notice" and so on. 
   They are named like the corresponding log levels in httpd_log.h. 
@@ -54,7 +87,7 @@
   It is distributed under a FreeBSD style license (if you want another license contact me)
   marc.battyani@fractalconcept.com
  
-Copyright 2000 Marc Battyani. All rights reserved. 
+Copyright 2000-2004 Marc Battyani.
 
 Redistribution and use in source and binary forms, with or without modification, are permitted provided 
 that the following conditions are met: 
@@ -175,6 +208,9 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSI
  * are handed a record that applies to the current location by implication or
  * inheritance, and modifying it will change the rules for other locations.
  */
+#define SERVER_IP_STR_LEN 20
+#define SERVER_ID_STR_LEN 100
+
 typedef struct excfg {
   int cmode;                  /* Environment to which record applies (directory,
 			       * server, or combination).
@@ -186,13 +222,18 @@ typedef struct excfg {
   int congenital;             /* Boolean: did we inherit an "Lisp"? */
   char *loc;                  /* Location to which this record applies. */
   int  DefaultLispServer;     // true if default values;
-  char LispServerIP[20];
+  char LispServerIP[SERVER_IP_STR_LEN];
   long LispServerPort;
-  char LispServerId[100];
-  long LispSocket;
-  long UnsafeLispSocket;
+  char LispServerId[SERVER_ID_STR_LEN];
 } excfg;
 
+
+static long LispSocket = 0;
+static long UnsafeLispSocket = 0;
+
+static char LispServerIP[SERVER_IP_STR_LEN];
+static long LispServerPort;
+
 pool *SocketPool = NULL;
 
 /*
@@ -371,13 +412,6 @@ static void setup_module_cells()
     if (lisp_pool == NULL) {
         lisp_pool = ap_make_sub_pool(NULL);
     };
-    /*
-     * Likewise for the table of routine/environment pairs we visit outside of
-     * request context.
-     */
-    if (static_calls_made == NULL) {
-        static_calls_made = ap_make_table(lisp_pool, 16);
-    };
 }
 #endif
 
@@ -421,21 +455,27 @@ int OpenLispSocket(excfg *cfg)
   int ret;
  
 #ifndef WIN32
-  if (cfg->LispSocket)
-    if (cfg->UnsafeLispSocket)
-      {
-	ap_pclosesocket(SocketPool, cfg->LispSocket);
-	cfg->LispSocket = 0;
-	cfg->UnsafeLispSocket = 0;
-      }
-    else
-      {
-	return cfg->LispSocket;
-      }
+  if (LispSocket)
+    {
+      if (UnsafeLispSocket || strcmp(cfg->LispServerIP, LispServerIP) || cfg->LispServerPort != LispServerPort)
+        {
+          ap_pclosesocket(SocketPool, LispSocket);
+          LispSocket = 0;
+          UnsafeLispSocket = 0;
+          LispServerIP[0] = 0;
+          LispServerPort = 0;
+        }
+      else
+        return LispSocket;
+    }
 #endif
 
-  cfg->LispSocket = 0;
-  cfg->UnsafeLispSocket = 0;
+  LispSocket = 0;
+  UnsafeLispSocket = 0;
+  strncpy(LispServerIP, cfg->LispServerIP, SERVER_IP_STR_LEN - 1);
+  LispServerIP[SERVER_IP_STR_LEN - 1] = 0;
+  LispServerPort = cfg->LispServerPort;
+
   addr.sin_addr.s_addr = inet_addr(cfg->LispServerIP);
   addr.sin_port = htons((unsigned short) cfg->LispServerPort);
   addr.sin_family = AF_INET;
@@ -459,7 +499,7 @@ int OpenLispSocket(excfg *cfg)
   if (ret == -1) 
     return -1;
 
-  cfg->LispSocket = sock;
+  LispSocket = sock;
 
   return sock;
 }
@@ -498,13 +538,15 @@ void CloseLispSocket(excfg *cfg, int Socket) // socket for WIN32
   if (Socket != -1)
     ap_pclosesocket(SocketPool, Socket);
 #else
-  if (!cfg->LispSocket)
+  if (!LispSocket)
     return;
 
-  ap_pclosesocket(SocketPool, cfg->LispSocket);
+  ap_pclosesocket(SocketPool, LispSocket);
 
-  cfg->LispSocket = 0;
-  cfg->UnsafeLispSocket = 0;
+  LispSocket = 0;
+  UnsafeLispSocket = 0;
+  LispServerIP[0] = 0;
+  LispServerPort = 0;
 #endif
 }
 
@@ -605,7 +647,7 @@ static int lisp_handler(request_rec *r)
     }
   ap_reset_timeout(r);
 
-  dcfg->UnsafeLispSocket = 1;
+  UnsafeLispSocket = 1;
 
   if (r->subprocess_env) 
     {
@@ -636,6 +678,8 @@ static int lisp_handler(request_rec *r)
 	    ret = SendLispHeaderLine(BuffSocket, "server-ip-port", elts[i].val);
 	  else if (!strcmp(elts[i].key, "SERVER_PROTOCOL"))
 	    ret = SendLispHeaderLine(BuffSocket, "server-protocol", elts[i].val);
+	  else if (!strcmp(elts[i].key, "SCRIPT_FILENAME"))
+	    ret = SendLispHeaderLine(BuffSocket, "script-filename", elts[i].val);
 	  else if (!strcmp(elts[i].key, "SSL_SESSION_ID"))
 	    ret = SendLispHeaderLine(BuffSocket, "ssl-session-id", elts[i].val);
 	  if (ret != 0) 
@@ -652,6 +696,8 @@ static int lisp_handler(request_rec *r)
    * without worrying about some joker sending a server-id header of
    * his own. (Robert Macomber)*/
   ret = SendLispHeaderLine(BuffSocket, "server-id", dcfg->LispServerId);
+  ret = SendLispHeaderLine(BuffSocket, "server-baseversion", SERVER_BASEVERSION);
+  ret = SendLispHeaderLine(BuffSocket, "modlisp-version", MOD_LISP_VERSION);
   if (ret!=0)
     {
       ap_kill_timeout(r);
@@ -669,7 +715,7 @@ static int lisp_handler(request_rec *r)
 	{
 	  if (!elts[i].key) continue;
 	  ret = SendLispHeaderLine(BuffSocket, 
-				   !strcmp(elts[i].key, "end")?"end-header":elts[i].key, 
+				   !strcasecmp(elts[i].key, "end")?"end-header":elts[i].key, 
 				   elts[i].val);
 	  if (ret!=0)
 	    {
@@ -731,7 +777,7 @@ static int lisp_handler(request_rec *r)
 	Header[l-1] = 0;
 
       ap_kill_timeout(r);
-      if (!strcmp(Header, "end"))
+      if (!strcasecmp(Header, "end"))
 	break;
       
       if (ap_bgets(Value, MAX_STRING_LEN, (BUFF *) BuffSocket) <= 0)
@@ -741,73 +787,69 @@ static int lisp_handler(request_rec *r)
       if (l > 0)
 	Value[l-1] = 0;
 
-      if (!strcmp(Header, "Content-Type"))
+      if (!strcasecmp(Header, "Content-Type"))
 	{
 	  char *tmp = ap_pstrdup(r->pool, Value);
 	  ap_content_type_tolower(tmp);
 	  r->content_type = tmp;
 	}
-      else if (!strcmp(Header, "Status"))
+      else if (!strcasecmp(Header, "Status"))
 	{
 	  r->status = atoi(Value);
 	  r->status_line = ap_pstrdup(r->pool, Value);
 	}
-      else if (!strcmp(Header, "Location"))
-	{
-	  ap_table_set(r->headers_out, Header, Value);
-	}
-      else if (!strcmp(Header, "Content-Length"))
+      else if (!strcasecmp(Header, "Content-Length"))
 	{
 	  ap_table_set(r->headers_out, Header, Value);
 	  ContentLength = atoi(Value);
 	}
-     else if (!strcmp(Header, "Last-Modified"))
+     else if (!strcasecmp(Header, "Last-Modified"))
 	{
 	  time_t mtime = ap_parseHTTPdate(Value);
 	  ap_update_mtime(r, mtime);
 	  ap_set_last_modified(r);
 	}
-     else if (!strcmp(Header, "Keep-Socket"))
+     else if (!strcasecmp(Header, "Keep-Socket"))
 	{
 	  KeepSocket = atoi(Value);
 	}
-      else if (!strcmp(Header, "Log-Emerg"))
+      else if (!strcasecmp(Header, "Log-Emerg"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_EMERG|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Alert"))
+      else if (!strcasecmp(Header, "Log-Alert"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_ALERT|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Crit"))
+      else if (!strcasecmp(Header, "Log-Crit"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_CRIT|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Error"))
+      else if (!strcasecmp(Header, "Log-Error"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_ERR|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Warning"))
+      else if (!strcasecmp(Header, "Log-Warning"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_WARNING|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Notice"))
+      else if (!strcasecmp(Header, "Log-Notice"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_NOTICE|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Info"))
+      else if (!strcasecmp(Header, "Log-Info"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_INFO|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log-Debug"))
+      else if (!strcasecmp(Header, "Log-Debug"))
 	{
 	  ap_log_error(APLOG_MARK, APLOG_DEBUG|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Log"))
+      else if (!strcasecmp(Header, "Log"))
 	{
 	  ap_log_error("mod_lisp", 0, APLOG_ERR|APLOG_NOERRNO, r->server, "%s", Value);
 	}
-      else if (!strcmp(Header, "Note"))
+      else if (!strcasecmp(Header, "Note"))
 	{
 	  char *pv = strchr(Value, ' ');
 	  if (pv != NULL)
@@ -816,7 +858,7 @@ static int lisp_handler(request_rec *r)
 	      ap_table_setn(r->notes, ap_pstrdup(r->pool, Value), ap_pstrdup(r->pool, pv));
 	    }
 	}
-      else if (!strcmp(Header, "Set-Cookie"))
+      else if (!strcasecmp(Header, "Set-Cookie"))
 	{
 	  ap_table_add(r->headers_out, Header, Value);
 	}
@@ -827,25 +869,22 @@ static int lisp_handler(request_rec *r)
   /* Send headers and data collected (if this was not a "header only" req. */
   ap_send_http_header(r);
   if (!r->header_only)
-    if (ContentLength > 0)
-      {
-	long ReadLength = ap_send_fb_length(BuffSocket, r, ContentLength);
-	if (ReadLength < ContentLength || r->connection->aborted)
-	  {
-	    char buffer[HUGE_STRING_LEN];
-	    ContentLength -= ReadLength;
-	    do
-	      {
-		ReadLength = ForceGets(buffer, (BUFF *) BuffSocket, 
-				       HUGE_STRING_LEN > ContentLength ? ContentLength : HUGE_STRING_LEN);
-		ContentLength -= ReadLength;
-	      }
-	    while (ReadLength > 0 && ContentLength > 0);
-	  }
-      }
+    {
+      if (ContentLength > 0)
+        {
+          long ReadLength = ap_send_fb_length(BuffSocket, r, ContentLength);
+          if (ReadLength < ContentLength || r->connection->aborted)
+            {
+              ap_log_error("mod_lisp", 0, APLOG_WARNING|APLOG_NOERRNO, r->server, "Could not send complete body to client, closing socket to Lisp");
+              ap_bclose(BuffSocket);
+              KeepSocket = 0;
+              LispSocket = 0;
+            }
+        }
       else
-	if (ContentLength == -1)
-	  ap_send_fb(BuffSocket, r);
+        if (ContentLength == -1)
+          ap_send_fb(BuffSocket, r);
+    }
 
 #ifdef WIN32
   KeepSocket = 0;
@@ -860,7 +899,8 @@ static int lisp_handler(request_rec *r)
       ap_bpushfd(BuffSocket, -1, -1); /* unlink buffer to keep socket */
       BuffSocket->flags &= ~B_SOCKET;
     }
-  dcfg->UnsafeLispSocket = 0;
+  UnsafeLispSocket = 0;
+
   return OK;
 }
 
@@ -907,10 +947,6 @@ static int lisp_handler(request_rec *r)
 static void lisp_init(server_rec *s, pool *p)
 {
     ap_add_version_component("mod_lisp/" MOD_LISP_VERSION);
-    /*
-     * Set up any module cells that ought to be initialised.
-     */
-    //    setup_module_cells();
 }
 
 /* 
@@ -928,11 +964,6 @@ static void lisp_init(server_rec *s, pool *p)
 static void lisp_child_init(server_rec *s, pool *p)
 {
     SocketPool = ap_make_sub_pool(NULL);
-
-    /*
-     * Set up any module cells that ought to be initialised.
-     */
-    //    setup_module_cells();
 }
 
 /* 
@@ -985,7 +1016,6 @@ static void *lisp_create_dir_config(pool *p, char *dirspec)
     strcpy(cfg->LispServerId, "apache");
     cfg->LispServerPort = 3000;
     cfg->DefaultLispServer = 1;
-    cfg->LispSocket = 0;
 
     cfg->cmode = CONFIG_MODE_DIRECTORY;
     dname = (dname != NULL) ? dname : "";
@@ -1059,7 +1089,6 @@ static void *lisp_merge_dir_config(pool *p, void *parent_conf,
 	  merged_config->DefaultLispServer = 1;
 	}
 
-    merged_config->LispSocket = 0;
     return (void *) merged_config;
 }
 
@@ -1087,7 +1116,6 @@ static void *lisp_create_server_config(pool *p, server_rec *s)
     strcpy(cfg->LispServerIP, "127.0.0.1");
     strcpy(cfg->LispServerId, "apache");
     cfg->LispServerPort = 3000;
-    cfg->LispSocket = 0;
     cfg->DefaultLispServer = 1;
     sname = (sname != NULL) ? sname : "";
     cfg->loc = ap_pstrcat(p, "SVR(", sname, ")", NULL);
@@ -1148,168 +1176,19 @@ static void *lisp_merge_server_config(pool *p, void *server1_conf,
 	  merged_config->DefaultLispServer = 1;
 	}
 
-    merged_config->LispSocket = 0;
     return (void *) merged_config;
 }
 
-/*
- * This routine is called after the request has been read but before any other
- * phases have been processed.  This allows us to make decisions based upon
- * the input header fields.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
- * further modules are called for this phase.
- */
-static int lisp_post_read_request(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine gives our module an opportunity to translate the URI into an
- * actual filename.  If we don't do anything special, the server's default
- * rules (Alias directives and the like) will continue to be followed.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
- * further modules are called for this phase.
- */
-static int lisp_translate_handler(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine is called to check the authentication information sent with
- * the request (such as looking up the user in a database and verifying that
- * the [encrypted] password sent matches the one in the database).
- *
- * The return value is OK, DECLINED, or some HTTP_mumble error (typically
- * HTTP_UNAUTHORIZED).  If we return OK, no other modules are given a chance
- * at the request during this phase.
- */
-static int lisp_check_user_id(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine is called to check to see if the resource being requested
- * requires authorisation.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
- * other modules are called during this phase.
- *
- * If *all* modules return DECLINED, the request is aborted with a server
- * error.
- */
-static int lisp_auth_checker(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine is called to check for any module-specific restrictions placed
- * upon the requested resource.  (See the mod_access module for an lisp.)
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  All modules with an
- * handler for this phase are called regardless of whether their predecessors
- * return OK or DECLINED.  The first one to return any other status, however,
- * will abort the sequence (and the request) as usual.
- */
-static int lisp_access_checker(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine is called to determine and/or set the various document type
- * information bits, like Content-type (via r->content_type), language, et
- * cetera.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, no
- * further modules are given a chance at the request for this phase.
- */
-static int lisp_type_checker(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine is called to perform any module-specific fixing of header
- * fields, et cetera.  It is invoked just before any content-handler.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, the
- * server will still call any remaining modules with an handler for this
- * phase.
- */
-static int lisp_fixer_upper(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return OK;
-}
-
-/*
- * This routine is called to perform any module-specific logging activities
- * over and above the normal server things.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, any
- * remaining modules with an handler for this phase will still be called.
- */
-static int lisp_logger(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
-/*
- * This routine is called to give the module a chance to look at the request
- * headers and take any appropriate specific actions early in the processing
- * sequence.
- *
- * The return value is OK, DECLINED, or HTTP_mumble.  If we return OK, any
- * remaining modules with handlers for this phase will still be called.
- */
-static int lisp_header_parser(request_rec *r)
-{
-    excfg *cfg;
-
-    cfg = our_dconfig(r);
-    return DECLINED;
-}
-
 static const char *SetLispServer(cmd_parms *cmd, void *mconfig, char *serverIp, char *port, char *serverId)
 {
   excfg *cfg = (excfg *) mconfig;
 
-  strncpy(cfg->LispServerIP, serverIp, 19);
-  cfg->LispServerIP[19] = 0;
-  strncpy(cfg->LispServerId, serverId, 99);
-  cfg->LispServerId[99] = 0;
+  strncpy(cfg->LispServerIP, serverIp, SERVER_IP_STR_LEN - 1);
+  cfg->LispServerIP[SERVER_IP_STR_LEN - 1] = 0;
+  strncpy(cfg->LispServerId, serverId, SERVER_ID_STR_LEN - 1);
+  cfg->LispServerId[SERVER_ID_STR_LEN - 1] = 0;
   cfg->LispServerPort = atoi(port);
   cfg->DefaultLispServer = 0;
-  cfg->LispSocket = 0;
 
   return NULL;
 }
@@ -1389,16 +1268,14 @@ module lisp_module =
     lisp_merge_server_config,        /* server config merger */
     lisp_cmds,               /* command table */
     lisp_handlers,           /* [7] list of handlers */
-    lisp_translate_handler,  /* [2] filename-to-URI translation */
-    lisp_check_user_id,      /* [5] check/validate user_id */
-    lisp_auth_checker,       /* [6] check user_id is valid *here* */
-    lisp_access_checker,     /* [4] check access by host address */
-    lisp_type_checker,       /* [7] MIME type checker/setter */
-    lisp_fixer_upper,        /* [8] fixups */
-    lisp_logger,             /* [10] logger */
-#if MODULE_MAGIC_NUMBER >= 19970103
-    lisp_header_parser,      /* [3] header parser */
-#endif
+    NULL,                    /* [2] filename-to-URI translation */
+    NULL,                    /* [5] check/validate user_id */
+    NULL,                    /* [6] check user_id is valid *here* */
+    NULL,                    /* [4] check access by host address */
+    NULL,                    /* [7] MIME type checker/setter */
+    NULL,                    /* [8] fixups */
+    NULL,                    /* [10] logger */
+    NULL,                    /* [3] header parser */
 #if MODULE_MAGIC_NUMBER >= 19970719
     lisp_child_init,         /* process initializer */
 #endif
@@ -1406,6 +1283,6 @@ module lisp_module =
     lisp_child_exit,         /* process exit/cleanup */
 #endif
 #if MODULE_MAGIC_NUMBER >= 19970902
-    lisp_post_read_request   /* [1] post read_request handling */
+    NULL                     /* [1] post read_request handling */
 #endif
 };
